'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var react = require('react');

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

var __assign = Object.assign || function __assign(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
    }
    return t;
};

var defaultState = function (defaultValue) {
    return {
        loading: defaultValue === undefined || defaultValue === null,
        value: defaultValue,
    };
};
var reducer = function (state, action) {
    switch (action.type) {
        case 'error':
            return __assign({}, state, { error: action.error, loading: false });
        case 'reset':
            return defaultState(action.defaultValue);
        case 'value':
            return __assign({}, state, { loading: false, value: action.value });
        default:
            return state;
    }
};
var useLoadingValue = (function (getDefaultValue) {
    var defaultValue = getDefaultValue ? getDefaultValue() : undefined;
    var _a = react.useReducer(reducer, defaultState(defaultValue)), state = _a[0], dispatch = _a[1];
    var reset = function () {
        var defaultValue = getDefaultValue ? getDefaultValue() : undefined;
        dispatch({ type: 'reset', defaultValue: defaultValue });
    };
    var setError = function (error) {
        dispatch({ type: 'error', error: error });
    };
    var setValue = function (value) {
        dispatch({ type: 'value', value: value });
    };
    return {
        error: state.error,
        loading: state.loading,
        reset: reset,
        setError: setError,
        setValue: setValue,
        value: state.value,
    };
});

var useComparatorRef = function (value, isEqual, onChange) {
    var ref = react.useRef(value);
    react.useEffect(function () {
        if (!isEqual(value, ref.current)) {
            ref.current = value;
            if (onChange) {
                onChange();
            }
        }
    });
    return ref;
};
var isEqual = function (v1, v2) {
    var bothNull = !v1 && !v2;
    var equal = !!v1 && !!v2 && v1.isEqual(v2);
    return bothNull || equal;
};
var useIsEqualRef = function (value, onChange) {
    return useComparatorRef(value, isEqual, onChange);
};

var useCollection = (function (query, options) {
    var _a = useLoadingValue(), error = _a.error, loading = _a.loading, reset = _a.reset, setError = _a.setError, setValue = _a.setValue, value = _a.value;
    var ref = useIsEqualRef(query, reset);
    react.useEffect(function () {
        if (!ref.current) {
            setValue(undefined);
            return;
        }
        var listener = options
            ? ref.current.onSnapshot(options, setValue, setError)
            : ref.current.onSnapshot(setValue, setError);
        return function () {
            listener();
        };
    }, [ref.current]);
    return {
        error: error,
        loading: loading,
        value: value,
    };
});

var snapshotToData = function (snapshot, idField) {
    return (__assign({}, snapshot.data(), (idField ? (_a = {}, _a[idField] = snapshot.id, _a) : null)));
    var _a;
};

var useCollectionData = (function (query, idField) {
    var _a = useCollection(query), error = _a.error, loading = _a.loading, value = _a.value;
    return {
        error: error,
        loading: loading,
        value: (value
            ? value.docs.map(function (doc) { return snapshotToData(doc, idField); })
            : undefined),
    };
});

var useDocument = (function (docRef, options) {
    var _a = useLoadingValue(), error = _a.error, loading = _a.loading, reset = _a.reset, setError = _a.setError, setValue = _a.setValue, value = _a.value;
    var ref = useIsEqualRef(docRef, reset);
    react.useEffect(function () {
        if (!ref.current) {
            setValue(undefined);
            return;
        }
        var listener = options
            ? ref.current.onSnapshot(options, setValue, setError)
            : ref.current.onSnapshot(setValue, setError);
        return function () {
            listener();
        };
    }, [ref.current]);
    return {
        error: error,
        loading: loading,
        value: value,
    };
});

var useDocumentData = (function (docRef, idField) {
    var _a = useDocument(docRef), error = _a.error, loading = _a.loading, value = _a.value;
    return {
        error: error,
        loading: loading,
        value: (value ? snapshotToData(value, idField) : undefined),
    };
});

exports.useCollection = useCollection;
exports.useCollectionData = useCollectionData;
exports.useDocument = useDocument;
exports.useDocumentData = useDocumentData;
